package 
{
	import com.bit101.components.CheckBox;
	import com.bit101.components.HUISlider;
	import com.bit101.components.VBox;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.geom.Rectangle;
	import flash.utils.getTimer;
	
	/**
	 * Test file for the PostionalSound class
	 * @author Damian Connolly
	 */
	public class Main extends Sprite 
	{
		/********************************************************************************/
		
		[Embed(source = "../bin/dave_brubeck_quartet_take_five.mp3")]
		private var m_soundSource:Class;
		
		private var m_posSound:PositionalSound 	= null;	// our positional sound class
		private var m_player:Player				= null;	// the player that we're using as a ref
		private var m_prevTime:int 				= 0;	// use to help calculate delta time
		
		/********************************************************************************/
		
		public function Main():void 
		{
			if ( this.stage != null ) this._init();
			else this.addEventListener( Event.ADDED_TO_STAGE, this._init );
		}
		
		/********************************************************************************/
		
		private function _init( e:Event = null ):void
		{
			this.removeEventListener( Event.ADDED_TO_STAGE, this._init );
			
			// create our player
			this.m_player 	= new Player;
			this.m_player.x	= this.stage.stageWidth * 0.5;
			this.m_player.y	= this.stage.stageHeight * 0.5;
			this.addChild( this.m_player );
			
			// create our sound
			this.m_posSound 			= new PositionalSound;
			this.m_posSound.refObj		= this.m_player;
			this.m_posSound.panType 	= PositionalSoundPanType.PLAYER;
			this.m_posSound.pos.setTo( this.stage.stageWidth * 0.5, this.stage.stageHeight * 0.5 );
			this.m_posSound.sound = new this.m_soundSource;										// creating from embed
			//this.m_posSound.load( new URLRequest( "dave_brubeck_quartet_take_five.mp3" ) );	// loading a sound
			this.m_posSound.play( 0, int.MAX_VALUE );
			
			this.m_posSound.innerAngle = 90.0;
			this.m_posSound.outerAngle = 120.0;
			this.m_posSound.rotation = 20.0;
			
			var r:Rectangle = new Rectangle( -50.0, -20.0, 100.0, 150.0 );
			this.m_posSound.clipRect = r;
			this.m_posSound.clipRectRotation = -70.0;
			
			// for updating
			this.addEventListener( Event.ENTER_FRAME, this._onEnterFrame );
			
			// create our gui so we can edit the sounds properties
			//this._createGUI();
		}
		
		// called every frame
		private function _onEnterFrame( e:Event ):void
		{
			// calculate delta time
			var currTime:int 	= getTimer();
			var dt:Number		= ( currTime - this.m_prevTime ) * 0.001;
			this.m_prevTime 	= currTime;
			
			// update our reference position and update the sound and player
			var mx:Number = this.stage.mouseX;
			var my:Number = this.stage.mouseY;
			this.m_player.update( dt, mx, my );
			this.m_posSound.update( dt );
			
			// draw our debug
			this.m_posSound.drawDebug( this.graphics, true );
		}
		
		/********************************************************************************/
		
		// creates our minimal comps gui
		private function _createGUI():void
		{
			// our container
			var vbox:VBox 	= new VBox( this, 10.0, 10.0 );
			vbox.spacing	= 4.0;
			
			// rotation
			var rotation:HUISlider = new HUISlider( vbox, 0.0, 0.0, "rotation", function( e:Event ):void
			{
				m_posSound.rotation = rotation.value;
			});
			rotation.maximum	= 360.0;
			rotation.value		= this.m_posSound.rotation;
			
			// inner radius
			var innerRadius:HUISlider = new HUISlider( vbox, 0.0, 0.0, "inner radius", function( e:Event ):void
			{
				m_posSound.innerRadius 	= innerRadius.value;
				outerRadius.value		= m_posSound.outerRadius; // inner radius changes outer radius if necessary
			});
			innerRadius.maximum = this.stage.stageHeight * 0.5;
			innerRadius.value	= this.m_posSound.innerRadius;
			
			// outer radius
			var outerRadius:HUISlider = new HUISlider( vbox, 0.0, 0.0, "outer radius", function( e:Event ):void
			{
				m_posSound.outerRadius 	= outerRadius.value;
				outerRadius.value		= m_posSound.outerRadius; // outer radius is limited by inner radius, so set in case we were clamped
			});
			outerRadius.maximum = this.stage.stageHeight * 0.5;
			outerRadius.value	= this.m_posSound.outerRadius;
			
			// inner angle
			var innerAngle:HUISlider = new HUISlider( vbox, 0.0, 0.0, "inner angle", function( e:Event ):void
			{
				m_posSound.innerAngle 	= innerAngle.value;
				outerAngle.value		= m_posSound.outerAngle; // inner angle changes outer angle if necessary
			});
			innerAngle.maximum 	= 360.0;
			innerAngle.value	= this.m_posSound.innerAngle;
			
			// outer angle
			var outerAngle:HUISlider = new HUISlider( vbox, 0.0, 0.0, "outer angle", function( e:Event ):void
			{
				m_posSound.outerAngle 	= outerAngle.value;
				outerAngle.value		= m_posSound.outerAngle; // outer angle is limited by outer radius, so set in case we were clamped
			});
			outerAngle.maximum 	= 360.0;
			outerAngle.value	= this.m_posSound.outerAngle;
			
			// volume
			var volume:HUISlider = new HUISlider( vbox, 0.0, 0.0, "max volume", function( e:Event ):void
			{
				m_posSound.volume 		= volume.value;
				outerRadiusVolume.value	= m_posSound.outerRadiusVolume; // volume changes outerRadiusVolume if necessary
			});
			volume.maximum 	= 1.0;
			volume.value	= this.m_posSound.volume;
			
			// outer radius volume
			var outerRadiusVolume:HUISlider = new HUISlider( vbox, 0.0, 0.0, "outer radius volume", function( e:Event ):void
			{
				m_posSound.outerRadiusVolume 	= outerRadiusVolume.value;
				outerRadiusVolume.value			= m_posSound.outerRadiusVolume; // outer radius volume is limited by volume, so set in case we were clamped
			});
			outerRadiusVolume.maximum	= 1.0;
			outerRadiusVolume.value		= this.m_posSound.outerRadiusVolume;
			
			// positional vol limit
			var posVolLimit:HUISlider = new HUISlider( vbox, 0.0, 0.0, "pos. vol behind limit", function( e:Event ):void
			{
				m_posSound.positionalVolBehindLimit = posVolLimit.value;
			});
			posVolLimit.maximum	= 1.0;
			posVolLimit.value 	= this.m_posSound.positionalVolBehindLimit;
			
			// positional vol mult
			var posVolMult:HUISlider = new HUISlider( vbox, 0.0, 0.0, "pos. vol behind mult", function( e:Event ):void
			{
				m_posSound.positionalVolBehindMult = posVolMult.value;
			});
			posVolMult.maximum 	= 1.0;
			posVolMult.value	= this.m_posSound.positionalVolBehindMult;
			
			// pan type none
			var panTypeNone:CheckBox = new CheckBox( vbox, 0.0, 0.0, "pan type: NONE", function( e:Event ):void
			{
				m_posSound.panType 		= PositionalSoundPanType.NONE;
				panTypePlayer.selected	= false;
				panTypeRef.selected		= false;
			});
			panTypeNone.selected = ( this.m_posSound.panType == PositionalSoundPanType.NONE );
			
			// pan type player
			var panTypePlayer:CheckBox = new CheckBox( vbox, 0.0, 0.0, "pan type: PLAYER", function( e:Event ):void
			{
				m_posSound.panType 		= ( panTypePlayer.selected ) ? PositionalSoundPanType.PLAYER : PositionalSoundPanType.NONE;
				panTypeNone.selected	= false;
				panTypeRef.selected		= false;
			});
			panTypePlayer.selected = ( this.m_posSound.panType == PositionalSoundPanType.PLAYER );
			
			// pan type reference
			var panTypeRef:CheckBox = new CheckBox( vbox, 0.0, 0.0, "pan type: REFERENCE", function( e:Event ):void
			{
				m_posSound.panType 		= ( panTypeRef.selected ) ? PositionalSoundPanType.REFERENCE : PositionalSoundPanType.NONE;
				panTypeNone.selected	= false;
				panTypePlayer.selected	= false;
			});
			panTypeRef.selected = ( this.m_posSound.panType == PositionalSoundPanType.REFERENCE );
			
			// clip rect active
			var clipRect:Rectangle 		= new Rectangle( -20.0, -20.0, 40.0, 40.0 );
			var hasClipRect:CheckBox 	= new CheckBox( vbox, 0.0, 0.0, "has clip rect", function( e:Event ):void
			{
				m_posSound.clipRect = ( hasClipRect.selected ) ? clipRect : null;
			});
			hasClipRect.selected = ( this.m_posSound.clipRect != null );
			
			// clip rect inversed
			var isClipRectInversed:CheckBox = new CheckBox( vbox, 0.0, 0.0, "clip rect is inversed", function( e:Event ):void
			{
				m_posSound.isClipRectInversed = isClipRectInversed.selected;
			});
			isClipRectInversed.selected = this.m_posSound.isClipRectInversed;
			
			// clip rect x
			var clipRectX:HUISlider = new HUISlider( vbox, 0.0, 0.0, "clip rect x", function( e:Event ):void
			{
				clipRect.x 			= clipRectX.value;
				m_posSound.clipRect	= clipRect;
			});
			clipRectX.minimum 	= -this.stage.stageWidth * 0.5;
			clipRectX.maximum	= this.stage.stageWidth * 0.5;
			clipRectX.value		= clipRect.x;
			
			// clip rect y
			var clipRectY:HUISlider = new HUISlider( vbox, 0.0, 0.0, "clip rect y", function( e:Event ):void
			{
				clipRect.y 			= clipRectY.value;
				m_posSound.clipRect	= clipRect;
			});
			clipRectY.minimum 	= -this.stage.stageHeight * 0.5;
			clipRectY.maximum	= this.stage.stageHeight * 0.5;
			clipRectY.value		= clipRect.y;
			
			// clip rect width
			var clipRectW:HUISlider = new HUISlider( vbox, 0.0, 0.0, "clip rect w", function( e:Event ):void
			{
				clipRect.width		= clipRectW.value;
				m_posSound.clipRect	= clipRect;
			});
			clipRectW.maximum	= this.stage.stageWidth;
			clipRectW.value		= clipRect.width;
			
			// clip rect height
			var clipRectH:HUISlider = new HUISlider( vbox, 0.0, 0.0, "clip rect h", function( e:Event ):void
			{
				clipRect.height		= clipRectH.value;
				m_posSound.clipRect	= clipRect;
			});
			clipRectH.maximum	= this.stage.stageHeight;
			clipRectH.value		= clipRect.height;
			
			// clip rect rotation
			var clipRectRot:HUISlider = new HUISlider( vbox, 0.0, 0.0, "clip rect rotation", function( e:Event ):void
			{
				m_posSound.clipRectRotation = clipRectRot.value;
			});
			clipRectRot.maximum = 360.0;
			clipRectRot.value	= this.m_posSound.clipRectRotation;
		}
		
	}
	
}