package  
{
	/**
	 * The different types of pan that we can have
	 * @author Damian Connolly
	 */
	public class PositionalSoundPanType 
	{
		
		/********************************************************************************/
		
		/**
		 * No pan is applied is applied to the sound
		 */
		public static const NONE:PositionalSoundPanType = new PositionalSoundPanType;
		
		/**
		 * The pan is applied so it's relative to the player playing the game; i.e. if the
		 * reference object/pos is on the left of the sound, the pan will be left
		 */
		public static const PLAYER:PositionalSoundPanType = new PositionalSoundPanType;
		
		/**
		 * The pan is applied so it's relative to the reference object/pos. If we are using
		 * a reference position, then this will behave the same as PLAYER. If we are using a
		 * reference object, then the pan will be relative to the object; i.e. if the object
		 * has the sound on the left, the pan will be on the left; if the object doesn't move,
		 * but rotates 180 degrees, then the pan will now be on the right
		 */
		public static const REFERENCE:PositionalSoundPanType = new PositionalSoundPanType;
		
		/********************************************************************************/
		
		/**
		 * Converts the PositionalSoundPanType to a String
		 * @param type The PositionalSoundPanType that we want to convert
		 * @return The PositionalSoundPanType as a String
		 */
		public static function toString( type:PositionalSoundPanType ):String
		{
			if ( type == PositionalSoundPanType.NONE )
				return "None";
			else if ( type == PositionalSoundPanType.PLAYER )
				return "Player";
			else if ( type == PositionalSoundPanType.REFERENCE )
				return "Reference";
			return "Unknown PositionalSoundPanType";
		}
		
	}

}